@@ -9,30 +9,16 @@
  * Copyright (c) 2007, Frank Warmerdam
  * Copyright (c) 2008-2012, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
 #include "gdal_priv.h"
 
 #include <algorithm>
+#include <cmath>
 #include <cstring>
+#include <utility>
 
 #include "cpl_conv.h"
 #include "cpl_error.h"
@@ -111,19 +97,27 @@ static GDALDataType GetWorkDataType(GDALDataType eDataType)
             eWrkDT = GDT_Byte;
             break;
 
+        case GDT_Int16:
+            eWrkDT = GDT_Int16;
+            break;
+
         case GDT_UInt16:
+            eWrkDT = GDT_UInt16;
+            break;
+
         case GDT_UInt32:
             eWrkDT = GDT_UInt32;
             break;
 
         case GDT_Int8:
-        case GDT_Int16:
         case GDT_Int32:
         case GDT_CInt16:
         case GDT_CInt32:
             eWrkDT = GDT_Int32;
             break;
 
+        case GDT_Float16:
+        case GDT_CFloat16:
         case GDT_Float32:
         case GDT_CFloat32:
             eWrkDT = GDT_Float32;
@@ -139,7 +133,8 @@ static GDALDataType GetWorkDataType(GDALDataType eDataType)
             eWrkDT = eDataType;
             break;
 
-        default:
+        case GDT_Unknown:
+        case GDT_TypeCount:
             CPLAssert(false);
             eWrkDT = GDT_Float64;
             break;
@@ -162,11 +157,20 @@ bool GDALNoDataMaskBand::IsNoDataInRange(double dfNoDataValue,
             return GDALIsValueInRange<GByte>(dfNoDataValue);
         }
 
+        case GDT_Int16:
+        {
+            return GDALIsValueInRange<GInt16>(dfNoDataValue);
+        }
+
+        case GDT_UInt16:
+        {
+            return GDALIsValueInRange<GUInt16>(dfNoDataValue);
+        }
+
         case GDT_UInt32:
         {
             return GDALIsValueInRange<GUInt32>(dfNoDataValue);
         }
-
         case GDT_Int32:
         {
             return GDALIsValueInRange<GInt32>(dfNoDataValue);
@@ -182,9 +186,15 @@ bool GDALNoDataMaskBand::IsNoDataInRange(double dfNoDataValue,
             return GDALIsValueInRange<int64_t>(dfNoDataValue);
         }
 
+        case GDT_Float16:
+        {
+            return std::isnan(dfNoDataValue) || std::isinf(dfNoDataValue) ||
+                   GDALIsValueInRange<GFloat16>(dfNoDataValue);
+        }
+
         case GDT_Float32:
         {
-            return CPLIsNan(dfNoDataValue) || CPLIsInf(dfNoDataValue) ||
+            return std::isnan(dfNoDataValue) || std::isinf(dfNoDataValue) ||
                    GDALIsValueInRange<float>(dfNoDataValue);
         }
 
@@ -224,6 +234,70 @@ CPLErr GDALNoDataMaskBand::IReadBlock(int nXBlockOff, int nYBlockOff,
                      nBlockXSize, &sExtraArg);
 }
 
+/************************************************************************/
+/*                            SetZeroOr255()                            */
+/************************************************************************/
+
+#if (defined(__GNUC__) && !defined(__clang__))
+__attribute__((optimize("tree-vectorize")))
+#endif
+static void
+SetZeroOr255(GByte *pabyDestAndSrc, size_t nBufSize, GByte byNoData)
+{
+    for (size_t i = 0; i < nBufSize; ++i)
+    {
+        pabyDestAndSrc[i] = (pabyDestAndSrc[i] == byNoData) ? 0 : 255;
+    }
+}
+
+template <class T>
+#if (defined(__GNUC__) && !defined(__clang__))
+__attribute__((optimize("tree-vectorize")))
+#endif
+static void
+SetZeroOr255(GByte *pabyDest, const T *panSrc, size_t nBufSize, T nNoData)
+{
+    for (size_t i = 0; i < nBufSize; ++i)
+    {
+        pabyDest[i] = (panSrc[i] == nNoData) ? 0 : 255;
+    }
+}
+
+template <class T>
+static void SetZeroOr255(GByte *pabyDest, const T *panSrc, int nBufXSize,
+                         int nBufYSize, GSpacing nPixelSpace,
+                         GSpacing nLineSpace, T nNoData)
+{
+    if (nPixelSpace == 1 && nLineSpace == nBufXSize)
+    {
+        const size_t nBufSize = static_cast<size_t>(nBufXSize) * nBufYSize;
+        SetZeroOr255(pabyDest, panSrc, nBufSize, nNoData);
+    }
+    else if (nPixelSpace == 1)
+    {
+        for (int iY = 0; iY < nBufYSize; iY++)
+        {
+            SetZeroOr255(pabyDest, panSrc, nBufXSize, nNoData);
+            pabyDest += nLineSpace;
+            panSrc += nBufXSize;
+        }
+    }
+    else
+    {
+        size_t i = 0;
+        for (int iY = 0; iY < nBufYSize; iY++)
+        {
+            GByte *pabyLineDest = pabyDest + iY * nLineSpace;
+            for (int iX = 0; iX < nBufXSize; iX++)
+            {
+                *pabyLineDest = (panSrc[i] == nNoData) ? 0 : 255;
+                ++i;
+                pabyLineDest += nPixelSpace;
+            }
+        }
+    }
+}
+
 /************************************************************************/
 /*                             IRasterIO()                              */
 /************************************************************************/
@@ -259,48 +333,85 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
         if (nPixelSpace == 1 && nLineSpace == nBufXSize)
         {
             const size_t nBufSize = static_cast<size_t>(nBufXSize) * nBufYSize;
-            for (size_t i = 0; i < nBufSize; ++i)
-            {
-                pabyData[i] = pabyData[i] == byNoData ? 0 : 255;
-            }
+            SetZeroOr255(pabyData, nBufSize, byNoData);
         }
         else
         {
-            for (int iY = 0; iY < nBufYSize; iY++)
-            {
-                GByte *pabyLine = pabyData + iY * nLineSpace;
-                for (int iX = 0; iX < nBufXSize; iX++)
-                {
-                    *pabyLine = *pabyLine == byNoData ? 0 : 255;
-                    pabyLine += nPixelSpace;
-                }
-            }
+            SetZeroOr255(pabyData, pabyData, nBufXSize, nBufYSize, nPixelSpace,
+                         nLineSpace, byNoData);
         }
         return CE_None;
     }
 
+    const auto AllocTempBufferOrFallback =
+        [this, eRWFlag, nXOff, nYOff, nXSize, nYSize, pData, nBufXSize,
+         nBufYSize, eBufType, nPixelSpace, nLineSpace,
+         psExtraArg](int nWrkDTSize) -> std::pair<CPLErr, void *>
+    {
+        auto poParentDS = m_poParent->GetDataset();
+        // Check if we must simulate a memory allocation failure
+        // Before checking the env variable, which is slightly expensive,
+        // check first for a special dataset name, which is a cheap test.
+        const char *pszOptVal =
+            poParentDS && strcmp(poParentDS->GetDescription(), "__debug__") == 0
+                ? CPLGetConfigOption(
+                      "GDAL_SIMUL_MEM_ALLOC_FAILURE_NODATA_MASK_BAND", "NO")
+                : "NO";
+        const bool bSimulMemAllocFailure =
+            EQUAL(pszOptVal, "ALWAYS") ||
+            (CPLTestBool(pszOptVal) &&
+             GDALMajorObject::GetMetadataItem(__func__, "__INTERNAL__") ==
+                 nullptr);
+        void *pTemp = nullptr;
+        if (!bSimulMemAllocFailure)
+        {
+            CPLErrorStateBackuper oErrorStateBackuper(CPLQuietErrorHandler);
+            pTemp = VSI_MALLOC3_VERBOSE(nWrkDTSize, nBufXSize, nBufYSize);
+        }
+        if (!pTemp)
+        {
+            const bool bAllocHasAlreadyFailed =
+                GDALMajorObject::GetMetadataItem(__func__, "__INTERNAL__") !=
+                nullptr;
+            CPLError(bAllocHasAlreadyFailed ? CE_Failure : CE_Warning,
+                     CPLE_OutOfMemory,
+                     "GDALNoDataMaskBand::IRasterIO(): cannot allocate %d x %d "
+                     "x %d bytes%s",
+                     nBufXSize, nBufYSize, nWrkDTSize,
+                     bAllocHasAlreadyFailed
+                         ? ""
+                         : ". Falling back to block-based approach");
+            if (bAllocHasAlreadyFailed)
+                return std::pair(CE_Failure, nullptr);
+            // Sets a metadata item to prevent potential infinite recursion
+            GDALMajorObject::SetMetadataItem(__func__, "IN", "__INTERNAL__");
+            const CPLErr eErr = GDALRasterBand::IRasterIO(
+                eRWFlag, nXOff, nYOff, nXSize, nYSize, pData, nBufXSize,
+                nBufYSize, eBufType, nPixelSpace, nLineSpace, psExtraArg);
+            GDALMajorObject::SetMetadataItem(__func__, nullptr, "__INTERNAL__");
+            return std::pair(eErr, nullptr);
+        }
+        return std::pair(CE_None, pTemp);
+    };
+
     if (eBufType == GDT_Byte)
     {
         const int nWrkDTSize = GDALGetDataTypeSizeBytes(eWrkDT);
-        void *pTemp = VSI_MALLOC3_VERBOSE(nWrkDTSize, nBufXSize, nBufYSize);
-        if (pTemp == nullptr)
-        {
-            return GDALRasterBand::IRasterIO(
-                eRWFlag, nXOff, nYOff, nXSize, nYSize, pTemp, nBufXSize,
-                nBufYSize, eWrkDT, nWrkDTSize, nBufXSize * nWrkDTSize,
-                psExtraArg);
-        }
+        auto [eErr, pTemp] = AllocTempBufferOrFallback(nWrkDTSize);
+        if (!pTemp)
+            return eErr;
 
-        const CPLErr eErr = m_poParent->RasterIO(
+        eErr = m_poParent->RasterIO(
             GF_Read, nXOff, nYOff, nXSize, nYSize, pTemp, nBufXSize, nBufYSize,
-            eWrkDT, nWrkDTSize, nBufXSize * nWrkDTSize, psExtraArg);
+            eWrkDT, nWrkDTSize, static_cast<GSpacing>(nBufXSize) * nWrkDTSize,
+            psExtraArg);
         if (eErr != CE_None)
         {
             VSIFree(pTemp);
             return eErr;
         }
 
-        const bool bIsNoDataNan = CPLIsNan(m_dfNoDataValue) != 0;
+        const bool bIsNoDataNan = std::isnan(m_dfNoDataValue) != 0;
         GByte *pabyDest = static_cast<GByte *>(pData);
 
         /* --------------------------------------------------------------------
@@ -310,41 +421,39 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
          */
         switch (eWrkDT)
         {
+            case GDT_Int16:
+            {
+                const auto nNoData = static_cast<int16_t>(m_dfNoDataValue);
+                const auto *panSrc = static_cast<const int16_t *>(pTemp);
+                SetZeroOr255(pabyDest, panSrc, nBufXSize, nBufYSize,
+                             nPixelSpace, nLineSpace, nNoData);
+            }
+            break;
+
+            case GDT_UInt16:
+            {
+                const auto nNoData = static_cast<uint16_t>(m_dfNoDataValue);
+                const auto *panSrc = static_cast<const uint16_t *>(pTemp);
+                SetZeroOr255(pabyDest, panSrc, nBufXSize, nBufYSize,
+                             nPixelSpace, nLineSpace, nNoData);
+            }
+            break;
+
             case GDT_UInt32:
             {
-                const GUInt32 nNoData = static_cast<GUInt32>(m_dfNoDataValue);
-                const GUInt32 *panSrc = static_cast<const GUInt32 *>(pTemp);
-
-                size_t i = 0;
-                for (int iY = 0; iY < nBufYSize; iY++)
-                {
-                    GByte *pabyLineDest = pabyDest + iY * nLineSpace;
-                    for (int iX = 0; iX < nBufXSize; iX++)
-                    {
-                        *pabyLineDest = panSrc[i] == nNoData ? 0 : 255;
-                        ++i;
-                        pabyLineDest += nPixelSpace;
-                    }
-                }
+                const auto nNoData = static_cast<GUInt32>(m_dfNoDataValue);
+                const auto *panSrc = static_cast<const GUInt32 *>(pTemp);
+                SetZeroOr255(pabyDest, panSrc, nBufXSize, nBufYSize,
+                             nPixelSpace, nLineSpace, nNoData);
             }
             break;
 
             case GDT_Int32:
             {
-                const GInt32 nNoData = static_cast<GInt32>(m_dfNoDataValue);
-                const GInt32 *panSrc = static_cast<const GInt32 *>(pTemp);
-
-                size_t i = 0;
-                for (int iY = 0; iY < nBufYSize; iY++)
-                {
-                    GByte *pabyLineDest = pabyDest + iY * nLineSpace;
-                    for (int iX = 0; iX < nBufXSize; iX++)
-                    {
-                        *pabyLineDest = panSrc[i] == nNoData ? 0 : 255;
-                        ++i;
-                        pabyLineDest += nPixelSpace;
-                    }
-                }
+                const auto nNoData = static_cast<GInt32>(m_dfNoDataValue);
+                const auto *panSrc = static_cast<const GInt32 *>(pTemp);
+                SetZeroOr255(pabyDest, panSrc, nBufXSize, nBufYSize,
+                             nPixelSpace, nLineSpace, nNoData);
             }
             break;
 
@@ -360,7 +469,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                     for (int iX = 0; iX < nBufXSize; iX++)
                     {
                         const float fVal = pafSrc[i];
-                        if (bIsNoDataNan && CPLIsNan(fVal))
+                        if (bIsNoDataNan && std::isnan(fVal))
                             *pabyLineDest = 0;
                         else if (ARE_REAL_EQUAL(fVal, fNoData))
                             *pabyLineDest = 0;
@@ -384,7 +493,7 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
                     for (int iX = 0; iX < nBufXSize; iX++)
                     {
                         const double dfVal = padfSrc[i];
-                        if (bIsNoDataNan && CPLIsNan(dfVal))
+                        if (bIsNoDataNan && std::isnan(dfVal))
                             *pabyLineDest = 0;
                         else if (ARE_REAL_EQUAL(dfVal, m_dfNoDataValue))
                             *pabyLineDest = 0;
@@ -400,44 +509,16 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
             case GDT_Int64:
             {
                 const auto *panSrc = static_cast<const int64_t *>(pTemp);
-
-                size_t i = 0;
-                for (int iY = 0; iY < nBufYSize; iY++)
-                {
-                    GByte *pabyLineDest = pabyDest + iY * nLineSpace;
-                    for (int iX = 0; iX < nBufXSize; iX++)
-                    {
-                        const auto nVal = panSrc[i];
-                        if (nVal == m_nNoDataValueInt64)
-                            *pabyLineDest = 0;
-                        else
-                            *pabyLineDest = 255;
-                        ++i;
-                        pabyLineDest += nPixelSpace;
-                    }
-                }
+                SetZeroOr255(pabyDest, panSrc, nBufXSize, nBufYSize,
+                             nPixelSpace, nLineSpace, m_nNoDataValueInt64);
             }
             break;
 
             case GDT_UInt64:
             {
                 const auto *panSrc = static_cast<const uint64_t *>(pTemp);
-
-                size_t i = 0;
-                for (int iY = 0; iY < nBufYSize; iY++)
-                {
-                    GByte *pabyLineDest = pabyDest + iY * nLineSpace;
-                    for (int iX = 0; iX < nBufXSize; iX++)
-                    {
-                        const auto nVal = panSrc[i];
-                        if (nVal == m_nNoDataValueUInt64)
-                            *pabyLineDest = 0;
-                        else
-                            *pabyLineDest = 255;
-                        ++i;
-                        pabyLineDest += nPixelSpace;
-                    }
-                }
+                SetZeroOr255(pabyDest, panSrc, nBufXSize, nBufYSize,
+                             nPixelSpace, nLineSpace, m_nNoDataValueUInt64);
             }
             break;
 
@@ -452,30 +533,41 @@ CPLErr GDALNoDataMaskBand::IRasterIO(GDALRWFlag eRWFlag, int nXOff, int nYOff,
 
     // Output buffer is non-Byte. Ask for Byte and expand to user requested
     // type
-    GByte *pabyBuf =
-        static_cast<GByte *>(VSI_MALLOC2_VERBOSE(nBufXSize, nBufYSize));
-    if (pabyBuf == nullptr)
-    {
-        return GDALRasterBand::IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize,
-                                         pData, nBufXSize, nBufYSize, eBufType,
-                                         nPixelSpace, nLineSpace, psExtraArg);
-    }
-    const CPLErr eErr =
-        IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize, pabyBuf, nBufXSize,
-                  nBufYSize, GDT_Byte, 1, nBufXSize, psExtraArg);
+    auto [eErr, pTemp] = AllocTempBufferOrFallback(sizeof(GByte));
+    if (!pTemp)
+        return eErr;
+
+    eErr = IRasterIO(eRWFlag, nXOff, nYOff, nXSize, nYSize, pTemp, nBufXSize,
+                     nBufYSize, GDT_Byte, 1, nBufXSize, psExtraArg);
     if (eErr != CE_None)
     {
-        VSIFree(pabyBuf);
+        VSIFree(pTemp);
         return eErr;
     }
 
     for (int iY = 0; iY < nBufYSize; iY++)
     {
-        GDALCopyWords(pabyBuf + static_cast<size_t>(iY) * nBufXSize, GDT_Byte,
-                      1, static_cast<GByte *>(pData) + iY * nLineSpace,
-                      eBufType, static_cast<int>(nPixelSpace), nBufXSize);
+        GDALCopyWords64(
+            static_cast<GByte *>(pTemp) + static_cast<size_t>(iY) * nBufXSize,
+            GDT_Byte, 1, static_cast<GByte *>(pData) + iY * nLineSpace,
+            eBufType, static_cast<int>(nPixelSpace), nBufXSize);
     }
-    VSIFree(pabyBuf);
+    VSIFree(pTemp);
     return CE_None;
 }
+
+/************************************************************************/
+/*                   EmitErrorMessageIfWriteNotSupported()              */
+/************************************************************************/
+
+bool GDALNoDataMaskBand::EmitErrorMessageIfWriteNotSupported(
+    const char *pszCaller) const
+{
+    ReportError(CE_Failure, CPLE_NoWriteAccess,
+                "%s: attempt to write to a nodata implicit mask band.",
+                pszCaller);
+
+    return true;
+}
+
 //! @endcond
